package com.ruoyi.common.filter;

import com.ruoyi.common.utils.DateUtils;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.context.i18n.LocaleContextHolder;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;
import java.util.*;

/**
 * request.parameter
 *
 * @author xiewd
 */
@Slf4j
public class ParameterRequestWrapper extends HttpServletRequestWrapper {

    private Map<String, String[]> params = new HashMap<>();

    /**
     * Constructs a request object wrapping the given request.
     *
     * @param request
     * @throws IllegalArgumentException if the request is null
     */
    public ParameterRequestWrapper(HttpServletRequest request) {
        super(request);
        //将参数表，赋予给当前的Map以便于持有request中的参数
        this.params.putAll(request.getParameterMap());
    }
    //region 拦截所有时间
//    public ParameterRequestWrapper(HttpServletRequest request) {
//        super(request);
//        //将参数表，赋予给当前的Map以便于持有request中的参数
//        final String key = "Zone";
//        String zone = request.getHeader(key);
//        // 获取所有参数的Map
//        Map<String, String[]> paramMap = request.getParameterMap();
//
//        for (Map.Entry<String, String[]> entry : paramMap.entrySet()) {
//            String paramNameEntry = entry.getKey();
//            String[] paramValuesEntry = entry.getValue();
//            log.info("Parameter Name: " + paramNameEntry);
//            for (String value : paramValuesEntry) {
//                log.info("Parameter Value: " + value);
//                //判断是否为时间，如果是时间，则变为UTC时区
//                Date date = DateUtils.parseDate(value);
//                if(!ObjectUtils.isEmpty(date)){
//                    String res =  DateUtils.UTCTime(value,zone);
//                    value = res;
//                }
//            }
//        }
//
//        this.params.putAll(paramMap);
//    }
    //endregion

    /**
     * 重载构造方法
     */

    public ParameterRequestWrapper(HttpServletRequest request, Map<String, Object> extendParams) {
        this(request);
        //这里将扩展参数写入参数表
        addAllParameters(extendParams);
    }

    /**
     * 在获取所有的参数名,必须重写此方法，否则对象中参数值映射不上
     *
     * @return
     */
    @Override
    public Enumeration<String> getParameterNames() {
        return new Vector(params.keySet()).elements();
    }

    /**
     * 重写getParameter方法
     *
     * @param name 参数名
     * @return 返回参数值
     */
    @Override
    public String getParameter(String name) {
        String[] values = params.get(name);
        if (values == null || values.length == 0) {
            return null;
        }
        return values[0];
    }

    @Override
    public String[] getParameterValues(String name) {
        String[] values = params.get(name);
        if (values == null || values.length == 0) {
            return null;
        }
        return values;
    }

    /**
     * 增加多个参数
     *
     * @param otherParams 增加的多个参数
     */
    public void addAllParameters(Map<String, Object> otherParams) {
        for (Map.Entry<String, Object> entry : otherParams.entrySet()) {
            addParameter(entry.getKey(), entry.getValue());
        }
    }

    /**
     * 增加参数
     *
     * @param name  参数名
     * @param value 参数值
     */
    public void addParameter(String name, Object value) {
        if (value != null) {
            if (value instanceof String[]) {
                params.put(name, (String[]) value);
            } else if (value instanceof String) {
                params.put(name, new String[]{(String) value});
            } else {
                params.put(name, new String[]{String.valueOf(value)});
            }
        }
    }

    public void test(HttpServletRequest request){
        // 在这里写下你想要对请求进行的修改逻辑
        // 获取单个参数值
        String paramName = request.getParameter("roleName");
        System.out.println("Parameter Value: " + paramName);

        // 获取参数值数组
        String[] paramValues = request.getParameterValues("paramName");
        if (paramValues != null) {
            for (String value : paramValues) {
                System.out.println("Parameter Value: " + value);
            }
        }

        // 获取所有参数的Map
        Map<String, String[]> paramMap = request.getParameterMap();
        for (Map.Entry<String, String[]> entry : paramMap.entrySet()) {
            String paramNameEntry = entry.getKey();
            String[] paramValuesEntry = entry.getValue();
            System.out.println("Parameter Name: " + paramNameEntry);
            for (String value : paramValuesEntry) {
                System.out.println("Parameter Value: " + value);
            }
        }
    }
}