package com.ruoyi.project.business.controller.contract;

import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.business.domain.TqContractLiftHistory;
import com.ruoyi.project.business.service.ITqContractLiftHistoryService;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.web.page.TableDataInfo;

/**
 * 合同电梯履历Controller
 * 
 * @author ruoyi
 * @date 2024-08-16
 */
@RestController
@RequestMapping("/business/liftHistory")
public class TqContractLiftHistoryController extends BaseController
{
    @Resource
    private ITqContractLiftHistoryService tqContractLiftHistoryService;

    /**
     * 查询合同电梯履历列表
     */
    // @PreAuthorize("@ss.hasPermi('system:history:list')")
    @GetMapping("/list")
    public TableDataInfo list(TqContractLiftHistory tqContractLiftHistory)
    {
        startPage();
        List<TqContractLiftHistory> list = tqContractLiftHistoryService.selectTqContractLiftHistoryList(tqContractLiftHistory);
        return getDataTable(list);
    }

    /**
     * 导出合同电梯履历列表
     */
    // @PreAuthorize("@ss.hasPermi('system:history:export')")
    @Log(title = "合同电梯履历", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TqContractLiftHistory tqContractLiftHistory)
    {
        List<TqContractLiftHistory> list = tqContractLiftHistoryService.selectTqContractLiftHistoryList(tqContractLiftHistory);
        ExcelUtil<TqContractLiftHistory> util = new ExcelUtil<TqContractLiftHistory>(TqContractLiftHistory.class);
        util.exportExcel(response, list, "合同电梯履历数据");
    }

    /**
     * 获取合同电梯履历详细信息
     */
    // @PreAuthorize("@ss.hasPermi('system:history:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(tqContractLiftHistoryService.selectTqContractLiftHistoryById(id));
    }

    /**
     * 新增合同电梯履历
     */
    // @PreAuthorize("@ss.hasPermi('system:history:add')")
    @Log(title = "合同电梯履历", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TqContractLiftHistory tqContractLiftHistory)
    {
        return toAjax(tqContractLiftHistoryService.insertTqContractLiftHistory(tqContractLiftHistory));
    }

    /**
     * 修改合同电梯履历
     */
    // @PreAuthorize("@ss.hasPermi('system:history:edit')")
    @Log(title = "合同电梯履历", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TqContractLiftHistory tqContractLiftHistory)
    {
        return toAjax(tqContractLiftHistoryService.updateTqContractLiftHistory(tqContractLiftHistory));
    }

    /**
     * 删除合同电梯履历
     */
    // @PreAuthorize("@ss.hasPermi('system:history:remove')")
    @Log(title = "合同电梯履历", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tqContractLiftHistoryService.deleteTqContractLiftHistoryByIds(ids));
    }
}
