package com.ruoyi.project.business.service.impl;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.ruoyi.common.enums.FileBuzType;
import com.ruoyi.common.enums.SerialNumberPrefix;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.*;
import com.ruoyi.common.utils.office.doc.CommonDocUtils;
import com.ruoyi.framework.redis.RedisCache;
import com.ruoyi.project.business.domain.TbCountry;
import com.ruoyi.project.business.domain.TbFile;
import com.ruoyi.project.business.domain.TqContract;
import com.ruoyi.project.business.domain.TqContractLift;
import com.ruoyi.project.business.domain.bo.TqContractBo;
import com.ruoyi.project.business.domain.bo.contract.TqContractLiftBo;
import com.ruoyi.project.business.domain.flow.Flow;
import com.ruoyi.project.business.domain.flow.FlowAuditParam;
import com.ruoyi.project.business.domain.vo.contract.TqContractDto;
import com.ruoyi.project.business.domain.vo.contract.TqContractExportVo;
import com.ruoyi.project.business.domain.vo.contract.TqContractLiftVo;
import com.ruoyi.project.business.domain.vo.contract.TqContractVo;
import com.ruoyi.project.business.domain.vo.currency.ContractCurrencyTime;
import com.ruoyi.project.business.dto.ContractSigningStatistic;
import com.ruoyi.project.business.dto.LiftCount;
import com.ruoyi.project.business.dto.LiftMonthCount;
import com.ruoyi.project.business.dto.TaskGenerateParams;
import com.ruoyi.project.business.enums.AuditFlowEnum;
import com.ruoyi.project.business.mapper.TbCountryMapper;
import com.ruoyi.project.business.mapper.TbFileMapper;
import com.ruoyi.project.business.mapper.contract.TqContractLiftHistoryMapper;
import com.ruoyi.project.business.mapper.contract.TqContractLiftMapper;
import com.ruoyi.project.business.mapper.contract.TqContractMapper;
import com.ruoyi.project.business.service.*;
import com.ruoyi.project.business.service.flow.IFlowService;
import com.ruoyi.project.system.domain.SysDictData;
import com.ruoyi.project.system.mapper.SysDictDataMapper;
import fr.opensagres.xdocreport.document.images.FileImageProvider;
import fr.opensagres.xdocreport.template.formatter.NullImageBehaviour;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.util.*;
import java.util.stream.Collectors;

import static com.ruoyi.common.utils.DecimalUtils.add;
import static com.ruoyi.project.business.enums.AuditFlowEnum.CONTRACT_FINANCE_NO;
import static com.ruoyi.project.business.enums.AuditFlowEnum.CONTRACT_FINANCE_YES;
import static java.math.BigDecimal.ZERO;
import static java.util.Objects.nonNull;
import static org.apache.commons.lang3.StringUtils.EMPTY;
import static org.apache.commons.lang3.StringUtils.isBlank;

/**
 * 合同列表Service业务层处理
 * 
 * @author ruoyi
 * @date 2024-08-13
 */
@Service
public class TqContractServiceImpl implements ITqContractService 
{
    @Resource
    private TqContractMapper tqContractMapper;
    @Resource
    private RedisCache redisCache;
    @Resource
    private TbFileMapper tbFileMapper;

    @Resource
    private TqContractLiftMapper tqContractLiftMapper;

    @Resource
    private ITbFileService tbFileService;

    @Resource
    private ITqContractLiftService tqContractLiftService;
    @Resource
    private ITqContractHistoryService tqContractHistoryService;

    @Resource
    IFlowService flowService;

    @Resource
    SysDictDataMapper sysDictDataMapper;

    public static final String APPROVE_STATUS_SUCCESS = "2";

    public static final String GENERATE_TASKS_FLAG = "1";
    @Resource
    TqContractLiftHistoryMapper tqContractLiftHistoryMapper;

    @Resource
    TbCountryMapper tbCountryMapper;

    @Resource
    ITbContractCurrencyService tbContractCurrencyService;

    public static final DecimalFormat MONEY = new DecimalFormat("#,###.##");

    /**
     * 查询合同列表
     * 
     * @param id 合同列表主键
     * @return 合同列表
     */
    @Override
    public TqContract selectTqContractById(Long id)
    {
        return tqContractMapper.selectTqContractById(id);
    }

    public TqContractVo getContractById(Long id){
        TqContractBo tqContractBo = new TqContractBo();
        tqContractBo.setId(id);
        TqContractVo returnTqContractBo = tqContractMapper.selectTqContractBoListApp(tqContractBo).get(0);
        //查询字典returnTqContractBo
        List<TbFile> annex = tbFileService.list(Wrappers.<TbFile>lambdaQuery().eq(TbFile::getBusinessId, id)
                .eq(TbFile::getBusinessType, FileBuzType.CONTRACT.getType()));
        if (nonNull(tqContractBo)) {
            returnTqContractBo.setFile(annex.stream().map(TbFile::getFilePath).collect(Collectors.joining(",")));
        } else {
            throw new ServiceException(MessageUtils.message("common.dataDeleted"));
        }
        return returnTqContractBo;

    }

    /**
     * 查询合同列表列表
     * 
     * @param tqContract 合同列表
     * @return 合同列表
     */
    @Override
    public List<TqContract> selectTqContractList(TqContract tqContract)
    {
        return tqContractMapper.selectTqContractList(tqContract);
    }

    public List<TqContractVo> selectTqContractBoList(TqContractBo tqContract){
        List<TqContractVo> tqContractVoList = tqContractMapper.selectTqContractBoList(tqContract);
        if(CollectionUtils.isNotEmpty(tqContractVoList)){
            List<Long> collect = tqContractVoList.stream().map(TqContractVo::getId)
                    .collect(Collectors.toList());
            Map<Long, Flow> flowMap = flowService.getFlowMap(collect, Arrays.asList(CONTRACT_FINANCE_YES, CONTRACT_FINANCE_NO));
            tqContractVoList.forEach(item -> {
                item.setFlow(flowMap.get(item.getId()));
            });
        }
        return tqContractVoList;
    }

    /**
     * @Author Chenj
     * @Description 将数据处理成导出需要的格式
     * @Date 11:00 2024/9/11
     **/
    @Override
    public List<TqContractExportVo> handleExportList(List<TqContractVo> tqContractVoList){
        List<TqContractExportVo> tqContractExportVoList = tqContractVoList.stream()
                .map(tqContractVo -> {
                    TqContractExportVo tqContractExportVo = new TqContractExportVo();
                    BeanUtils.copyProperties(tqContractVo,tqContractExportVo);
                    tqContractExportVo.setAmountExport(formatBigDecimal(tqContractExportVo.getAmount(),tqContractExportVo.getUnit()));
                    tqContractExportVo.setAgentName(tqContractExportVo.getAgentName() != null ?tqContractExportVo.getAgentName():MessageUtils.message("agent.headquarters"));
                    return tqContractExportVo;
                }).collect(Collectors.toList());

        return tqContractExportVoList;
    }
    @Override
    public List<TqContractVo> selectTqContractBoListApp(TqContractBo tqContract){
        tqContract.setUserId(SecurityUtils.getUserId());
        List<TqContractVo> tqContractVoList = tqContractMapper.selectTqContractBoListApp(tqContract);
        return tqContractVoList;
    }
    /**
     * 新增合同列表
     * 
     * @param tqContractDto 合同列表
     * @return 结果
     */
    @Override
    @Transactional
    public int insertTqContract(TqContractDto tqContractDto)
    {
        //补充创建日期和权限
        tqContractDto.setCreateTime(DateUtils.getNowDate());
        tqContractDto.setPermissionChar(SecurityUtils.getPermissionCharIns());
        //获取国家缩写
        TbCountry tbCountry = tbCountryMapper.selectTbCountryByCountryId(tqContractDto.getCountryId());
        if(ObjectUtils.isEmpty(tbCountry)){
            throw new ServiceException("未找到该国家的信息！");
        }
        //将前缀和国家信息补充完整
        String contractNo = redisCache.generateSerialNumberNew(SerialNumberPrefix.C.getPrefix(), tbCountry.getCountryAbbr());
        tqContractDto.setContractNo(contractNo);
        //根据币种获取签单时币种汇率
        List<ContractCurrencyTime> contractCurrencyTimeList = new ArrayList<>();
        contractCurrencyTimeList.add(new ContractCurrencyTime(
                Long.valueOf(tqContractDto.getCurrencyId()),
                tqContractDto.getSigningDate()
        ));
        //将合同的初始version设置为1
        tqContractDto.setVersion(1L);

        //将合同的
        tqContractMapper.insertTqContract(tqContractDto);
        //附件上传
        tbFileService.saveOrUpdateFile(tqContractDto.getFile(),tqContractDto.getId(),FileBuzType.CONTRACT.getType());
        // 梯形 子表   采用批量导入的形式
        List<TqContractLift> tqContractLiftList = tqContractDto.getTqContractLiftList();
        if(CollectionUtils.isNotEmpty(tqContractLiftList)){
            //补充完整信息
            tqContractLiftList = attachTqContractLift(tqContractDto,tqContractLiftList);
            tqContractLiftMapper.insertTqContractLiftBatch(tqContractLiftList);
        }
        //生成对应审批流
        this.generateTasks(tqContractDto);
        return 1;
    }

    @Override
    public String selectApproveStatus(TqContractDto tqContractDto){
        TqContractLiftBo tqContractLiftBo = new TqContractLiftBo();
        tqContractLiftBo.setVersion(tqContractDto.getVersion());
        tqContractLiftBo.setContractNo(tqContractDto.getContractNo());
        List<TqContractLiftVo> tqContractLiftVoList = tqContractLiftService.selectTqContractLiftList(tqContractLiftBo);
        if(CollectionUtils.isNotEmpty(tqContractLiftVoList)){
            if("toVoid".equals(tqContractLiftVoList.get(0).getType())){
                //作废
                return "4";
            }else{
                //通过
                return "2";
            }
        }
        return "2";
    }
    /**
     * 修改合同列表
     * 
     * @param tqContractDto 合同列表
     * @return 结果
     */
    @Override
    @Transactional
    public int updateTqContract(TqContractDto tqContractDto)
    {
        List<TqContractLift> tqContractLiftList = tqContractDto.getTqContractLiftList();
        String type = tqContractDto.getType();
        //如果是变更或者作废   将版本+1
        if("amendment".equals(type) || "toVoid".equals(type)){
            tqContractDto.setVersion(tqContractDto.getVersion() + 1L);
        }
        //如果是变更的数据  先将所有的 设备数据的id清空
        if("amendment".equals(type)){
            List<TqContractLift> liftList = tqContractDto.getTqContractLiftList();
            if (liftList != null) {
                liftList.forEach(item -> item.setId(null));
            }
            tqContractDto.setTqContractLiftList(liftList);
        }
        if(CollectionUtils.isNotEmpty(tqContractLiftList)){
            //获取设备更新List
            List<TqContractLift> tqContractLiftUpdateList = tqContractLiftList.stream().filter(item-> item.getId() != null).collect(Collectors.toList());
            List<Long> idList = tqContractLiftUpdateList.stream().map(item->item.getId()).collect(Collectors.toList());
            //如果是变更    就不存在删除   只有新增
            if("amendment".equals(type)){
                //将现在的
            }else{
                //删除当前合同下不存在的设备番号
                if(CollectionUtils.isNotEmpty(idList)){
                    tqContractLiftMapper.deleteTqContractLiftByNoId(tqContractDto.getContractNo(),tqContractDto.getVersion(),idList);
                }
            }
            //获取update数据
            if(CollectionUtils.isNotEmpty(tqContractLiftUpdateList)){
                tqContractLiftUpdateList = attachTqContractLift(tqContractDto,tqContractLiftUpdateList);
                tqContractLiftMapper.updateTqContractLiftBatch(tqContractLiftUpdateList);
            }
            //获取设备新增List
            List<TqContractLift> tqContractLiftAddList = tqContractLiftList.stream().filter(item-> item.getId() == null).collect(Collectors.toList());
            if(CollectionUtils.isNotEmpty(tqContractLiftAddList)){
                tqContractLiftAddList = attachTqContractLift(tqContractDto,tqContractLiftAddList);
                tqContractLiftMapper.insertTqContractLiftBatch(tqContractLiftAddList);
            }
        }
        tqContractDto.setUpdateTime(DateUtils.getNowDate());
        if(tqContractDto.getNeedFileFlag() != null && "1".equals(tqContractDto.getNeedFileFlag())){
            //附件上传
            tbFileService.saveOrUpdateFile(tqContractDto.getFile(), tqContractDto.getId(),FileBuzType.CONTRACT.getType());
        }

        //附件上传完毕后  新增履历表(存在履历信息就上传履历表)
        if(ObjectUtils.isNotEmpty(tqContractDto.getTqContractHistory())){
            tqContractHistoryService.insertTqContractHistory(tqContractDto.getTqContractHistory());
        }
        //在合同变更或者编辑页面操作的   （设备的审批流要重新恢复到status的初始状态）audit_status
        if(tqContractDto.getGenerateTasksFlag() != null){
            this.generateTasks(tqContractDto);
        }
        return tqContractMapper.updateTqContract(tqContractDto);
    }

     /**
      * @Author Chenj
      * @Description 将导出的价格格式化  货币开头    三位一逗且保留两位小数
      * @Date 15:56 2024/9/14
      **/
    public static String formatBigDecimal(BigDecimal value,String unit) {
        if(value != null){
            DecimalFormat decimalFormat = new DecimalFormat("#,###.00");
            return unit+decimalFormat.format(value);
        }
        return unit+"0.00";

    }

    private void generateTasks(TqContractDto tqContractDto){
        //变更或者作废（版本大于1） ||付款方式是信用证  || 付款比例不是是30/70  走财务流程
        AuditFlowEnum contract = null;
        contract =tqContractDto.getVersion() > 1|| "1".equals(tqContractDto.getPaymentMethod()) || !"1".equals(tqContractDto.getPaymentRatio())? CONTRACT_FINANCE_YES : CONTRACT_FINANCE_NO;
        //删除不需要的流程
        flowService.remove("1".equals(tqContractDto.getPaymentMethod()) || !"1".equals(tqContractDto.getPaymentRatio())? CONTRACT_FINANCE_YES : CONTRACT_FINANCE_NO, Collections.singletonList(tqContractDto.getId()));
        //判断是不是变更或者编辑界面操作的数据
        if(GENERATE_TASKS_FLAG.equals(tqContractDto.getGenerateTasksFlag())){
            flowService.generateTasks(
                    new TaskGenerateParams(contract, tqContractDto, SecurityUtils.getUserId(),
                            tqContractDto.getContractNo()));
        }
        //手动删除同一流程下多余的数据
    }
    private void updateFlow(TqContract tqContract){
        FlowAuditParam taskGenerateParams = new FlowAuditParam();
        taskGenerateParams.setAuditFlow(3L);
        taskGenerateParams.setBusinessId(tqContract.getId());
        taskGenerateParams.setStatus(0);
        taskGenerateParams.setRemark("重新审批");
        flowService.audit(taskGenerateParams);
    }

     /**
      * @Author Chenj
      * @Description 将合同信息补充完整
      * @Date 9:13 2024/8/21
      **/
    private List<TqContractLift>  attachTqContractLift(TqContractDto tqContractDto, List<TqContractLift>  tqContractLiftList){
        //根据排产交期查询汇率
        List<ContractCurrencyTime> scheduleList = tqContractLiftList.stream().filter(tqContractLift -> tqContractLift.getProductionScheduleDate() != null) // Filter out null signing dates
                .map(tqContractLift -> new ContractCurrencyTime(
                        Long.parseLong(tqContractLift.getCurrencyId()), tqContractLift.getProductionScheduleDate()
                ))
                .collect(Collectors.toList());
        Map<Long, BigDecimal> scheduleListIdAndRateMap = tbContractCurrencyService.selectExchangeRateByCurrencyId(scheduleList);

        //根据实际发货排产日期查询汇率
        List<ContractCurrencyTime> deliveryList = tqContractLiftList.stream().filter(tqContractLift -> tqContractLift.getActualDeliveryDate() != null) // Filter out null signing dates
                .map(tqContractLift -> new ContractCurrencyTime(
                        Long.parseLong(tqContractLift.getCurrencyId()), tqContractLift.getActualDeliveryDate()
                ))
                .collect(Collectors.toList());
        Map<Long, BigDecimal> deliveryListIdAndRateMap = tbContractCurrencyService.selectExchangeRateByCurrencyId(deliveryList);


        //根据签单日期查询汇率
        List<ContractCurrencyTime> signingDateList = tqContractLiftList.stream()
                .map(tqContractLift -> new ContractCurrencyTime(
                        Long.parseLong(tqContractLift.getCurrencyId()), tqContractDto.getSigningDate()
                ))
                .collect(Collectors.toList());
        Map<Long, BigDecimal> signingDateListIdAndRateMap = tbContractCurrencyService.selectExchangeRateByCurrencyId(signingDateList);

        tqContractLiftList.forEach(item -> {
            item.setContractNo(tqContractDto.getContractNo());
            item.setContractName(tqContractDto.getContractName());
            item.setAreaId(tqContractDto.getAreaId());
            item.setCountryId(tqContractDto.getCountryId());
            item.setCity(tqContractDto.getCity());
            item.setSalesman(tqContractDto.getSalesman());
            item.setSalesModel(tqContractDto.getSalesModel());
            item.setEndCustomers(tqContractDto.getEndCustomers());
            item.setTradeTerms(tqContractDto.getTradeTerms());
            item.setAgentId(tqContractDto.getAgentId());
            item.setPaymentMethod(tqContractDto.getPaymentMethod());
            item.setAmountOfMoney(tqContractDto.getAmount());
            item.setSigningDate(tqContractDto.getSigningDate());
            item.setContractAmendmentDate(new Date());
            item.setPaymentRatio(tqContractDto.getPaymentRatio());
            item.setContractNoVersionNo(tqContractDto.getContractNo()+"-"+tqContractDto.getVersion());
            item.setPermissionChar(SecurityUtils.getPermissionCharIns());
            item.setType(tqContractDto.getType());
            item.setCollectionRatio(countCollectionRatio(item.getReceivedAmount(),item.getAccountsReceivableBalance()));
            //将签单币种汇率赋值
            if(scheduleListIdAndRateMap.containsKey(Long.parseLong(item.getCurrencyId()))){
                item.setRate4ProductionScheduling(scheduleListIdAndRateMap.get(Long.parseLong(item.getCurrencyId())));
            }
            //发货汇率
            if(deliveryListIdAndRateMap.containsKey(Long.parseLong(item.getCurrencyId()))){
                item.setRate4Shipment(deliveryListIdAndRateMap.get(Long.parseLong(item.getCurrencyId())));
            }
            //签单汇率
            if(signingDateListIdAndRateMap.containsKey(Long.parseLong(item.getCurrencyId()))){
                item.setRate4SigningContracts(signingDateListIdAndRateMap.get(Long.parseLong(item.getCurrencyId())));
            }
        });
        return tqContractLiftList;
    }

    public static String countCollectionRatio(BigDecimal a,BigDecimal b) {
        //只有两个金额都在才做计算
        if(a == null || b== null){
            return null;
        }
        // 计算百分比
        BigDecimal percentage = a.divide(b, 4, RoundingMode.HALF_UP).multiply(new BigDecimal("100"));
        // 保留两位小数并格式化为字符串
        String formattedPercentage = percentage.setScale(2, RoundingMode.HALF_UP).toPlainString() + "%";
        return formattedPercentage;
    }


    /**
     * 批量删除合同列表
     * 
     * @param ids 需要删除的合同列表主键
     * @return 结果
     */
    @Override
    public int deleteTqContractByIds(Long[] ids)
    {
        return tqContractMapper.deleteTqContractByIds(ids);
    }

    /**
     * 删除合同列表信息
     * 
     * @param id 合同列表主键
     * @return 结果
     */
    @Override
    public int deleteTqContractById(Long id)
    {
        return tqContractMapper.deleteTqContractById(id);
    }

    @Transactional
    @Override
    public void exportContract(Long id){
        //1.获取基础数据
        //1.1合同
        TqContract tqContract = this.getContractById(id);
        List<JSONObject> contractLift = Lists.newArrayList();
        //1.2电梯
        TqContractLiftBo tqContractLiftBo = new TqContractLiftBo();
        tqContractLiftBo.setContractNo(tqContract.getContractNo());
        List<TqContractLiftVo> tqContractLiftList = tqContractLiftMapper.selectTqContractLiftList(tqContractLiftBo);
        //货币单位
        SysDictData sysDictData =  sysDictDataMapper.selectSysDictDataByTypeAndValue("currency_type",tqContract.getCurrencyId());
        if(ObjectUtils.isEmpty(sysDictData) && StringUtils.isEmpty(sysDictData.getRemark())){
            throw new ServiceException("未找到该合同的单位");
        }
        String currencyUnit =  String.valueOf(sysDictData.getRemark().charAt(sysDictData.getRemark().length() - 1));

        // 2. 转换数据格式
        int i = 1;
        BigDecimal totalPrice = new BigDecimal(BigDecimal.ROUND_UP);
        for (TqContractLiftVo item : tqContractLiftList) {
            JSONObject data = new JSONObject();
            data.put("index",i);
            data.put("elevatorSpecifie", item.getProjectName()+item.getSpecification());
            data.put("unitPrice",currencyUnit + item.getUnitPrice());
            data.put("count",item.getCount());
            data.put("elvPrice",currencyUnit + item.getUnitPrice().multiply(BigDecimal.valueOf(item.getCount())));
            totalPrice  = totalPrice.add(item.getUnitPrice().multiply(BigDecimal.valueOf(item.getCount())));
            contractLift.add(data);
            i++;
        }
        Map<String, Object> map = Maps.newHashMap();
        map.put("buyers",tqContract.getContractName());
        map.put("address",tqContract.getAddress());
        map.put("telephone",tqContract.getShippingMark());
        map.put("totalPrice",currencyUnit +totalPrice);
        map.put("email",tqContract.getEmail());
        map.put("destinationPort",tqContract.getDestinationPort());
        map.put("shippingMark",tqContract.getShippingMark());
        map.put("shipmentPort",tqContract.getShipmentPort());
        map.put("shipmentDeadline", DateUtils.parseDateToStr("yyyy-MM-dd", tqContract.getShipmentDeadline()));
        map.put("contractLift", contractLift);


//        String path = CollectionUtils.isNotEmpty(previewList) ? RuoYiConfig.getProfile() + StringUtils.substringAfter(previewList.get(0).getFilePath(), Constants.RESOURCE_PREFIX): "/null";
//        FileImageProvider file = new FileImageProvider(new File(path), true);
        //只有审核通过的才会盖章
        if(APPROVE_STATUS_SUCCESS.equals(tqContract.getApproveStatus())){
            String path = "src/main/resources/wordTemplate/章.png";
            FileImageProvider file = new FileImageProvider(new File(path), true);
            file.setBehaviour(NullImageBehaviour.RemoveImageTemplate);
            file.setWidth(100f);
            file.setResize(true);
            map.put("photoSeal", file);
        }

        // 3. 填充模板并返回
        new CommonDocUtils()
                .loadFromClasspath("/wordTemplate/国际部销售合同模版2024.docx")
                .replaceData(map)
                .downloadDocx();

    }

    @Override
    public ContractSigningStatistic contractSigningStatistic(TqContractDto params) {
        String begin = params.getBeginTime();
        String end = params.getEndTime();
        if (isBlank(begin) || isBlank(end)) {
            throw new ServiceException("时间范围不能为空");
        }

        ContractSigningStatistic result = new ContractSigningStatistic();
        int year = DateUtils.getYear(end);
        int week = DateUtils.getWeekNumber(end);
        result.setYear(year);
        result.setWeek(week);
        // 1. 各区域本周、月电梯（签单、排产、发货）数量统计
        List<LiftCount> weekCountList = tqContractLiftMapper.countStatistic(week);
        List<List<String>> weekCountStatistics = Lists.newArrayList();
        weekCountStatistics.add(Lists.newArrayList("区域", "签单", "排产", "发货"));
        weekCountStatistics.addAll(getTotalList(weekCountList, "周合计"));
        result.setAreaCountStatisticsTitle(week + "周各区域数据");
        result.setAreaCountStatistics(weekCountStatistics);

        // 2. 全年月度（签单、排产、发货）数量统计
        List<LiftCount> monthCountListAll = tqContractLiftMapper.countStatisticByMonthAll(year);
        String month = DateUtils.getMonth(end).toString();
        LiftCount monthCount = monthCountListAll.stream().filter(item -> item.getKey().equals(month))
                .findFirst().orElse(new LiftCount(month, ZERO, ZERO, ZERO));
        weekCountStatistics.add(Lists.newArrayList("月合计",
                MONEY.format(monthCount.getSignCount()), MONEY.format(monthCount.getProductionCount()), MONEY.format(monthCount.getDeliveryCount())));

        List<List<String>> monthCountStatistics = Lists.newArrayList();
        monthCountStatistics.add(Lists.newArrayList("月份", "签约合计", "排产合计", "发货合计"));
        monthCountStatistics.addAll(getTotalList(monthCountListAll, "合计"));
        result.setMonthCountStatisticsTitle(year + "国际月度签约、排产、发货");
        result.setMonthCountStatistics(monthCountStatistics);

        // 3. 全年周（签单、排产、发货）数量统计
        List<LiftCount> weekCountListAll = tqContractLiftMapper.countStatisticByWeekAll(year);
        List<List<String>> weekCountStatisticsAll = Lists.newArrayList();
        weekCountStatisticsAll.add(Lists.newArrayList("周", "签约合计", "排产合计", "发货合计"));
        weekCountStatisticsAll.addAll(getTotalList(weekCountListAll, "合计"));
        result.setWeekCountStatisticsTitle(year + "国际每周签约、排产、发货数据");
        result.setWeekCountStatistics(weekCountStatisticsAll);

        // 4. 全年各区域签约金额
        List<LiftMonthCount> signingAmountAll = tqContractLiftMapper.countSigningStatisticAll(year);
        result.setSigningAreaColStatisticsTitle(year + "国际各区域签单");
        result.setSigningAreaColStatistics(getAreaColList(signingAmountAll));
        // 5. 全年各区域排产金额
        List<LiftMonthCount> productionAmountAll = tqContractLiftMapper.countProductionStatisticAll(year);
        result.setProductionAreaColStatisticsTitle(year + "国际各区域排产");
        result.setProductionAreaColStatistics(getAreaColList(productionAmountAll));
        // 6. 全年各区域发货金额
        List<LiftMonthCount> deliveryAmountAll = tqContractLiftMapper.countDeliveryStatisticAll(year);
        result.setDeliveryMonthColStatisticsTitle(year + "发货金额");
        result.setDeliveryMonthColStatistics(getMonthColList(deliveryAmountAll));
        // 7. 全年各区域排产金额
        result.setProductionMonthColStatisticsTitle(year + "排产金额");
        result.setProductionMonthColStatistics(getMonthColList(productionAmountAll));

        return result;
    }

    private static List<List<String>> getTotalList(List<LiftCount> countListAll, String totalTitle) {
        List<List<String>> result = Lists.newArrayList();
        BigDecimal signCountTotal = ZERO;
        BigDecimal productionCountTotal = ZERO;
        BigDecimal deliveryCountTotal = ZERO;
        for (LiftCount item : countListAll) {
            result.add(Lists.newArrayList(item.getKey(),
                    MONEY.format(item.getSignCount()), MONEY.format(item.getProductionCount()), MONEY.format(item.getDeliveryCount())));
            signCountTotal = add(signCountTotal, item.getSignCount());
            productionCountTotal = add(productionCountTotal, item.getProductionCount());
            deliveryCountTotal = add(deliveryCountTotal, item.getDeliveryCount());
        }
        result.add(Lists.newArrayList(totalTitle,
                MONEY.format(signCountTotal), MONEY.format(productionCountTotal), MONEY.format(deliveryCountTotal)));
        return result;
    }

    private List<List<String>> getAreaColList(List<LiftMonthCount> amountAll) {
        List<List<String>> result = Lists.newArrayList();
        if (CollectionUtils.isEmpty(amountAll)){
            return result;
        }

        List<String> areaColHeader = Lists.newArrayList("月份");
        result.add(areaColHeader);

        // 底部区域合计行
        List<String> areaSum = Lists.newArrayList("合计");
        amountAll.forEach(item -> {
            areaColHeader.add(item.getKey());
            // 1. 添加各区域每月的数据
            for (int i = 1; i <= 12; i++) {
                List<String> monthAreaCount;
                if (i < result.size()){
                    monthAreaCount = result.get(i);
                } else {
                    monthAreaCount = Lists.newArrayList(i + "月");
                    result.add(monthAreaCount);
                }
                BigDecimal current = ZERO;
                switch (i) {
                    case 1:
                        current = item.getJanuary();
                        break;
                    case 2:
                        current = item.getFebruary();
                        break;
                    case 3:
                        current = item.getMarch();
                        break;
                    case 4:
                        current = item.getApril();
                        break;
                    case 5:
                        current = item.getMay();
                        break;
                    case 6:
                        current = item.getJune();
                        break;
                    case 7:
                        current = item.getJuly();
                        break;
                    case 8:
                        current = item.getAugust();
                        break;
                    case 9:
                        current = item.getSeptember();
                        break;
                    case 10:
                        current = item.getOctober();
                        break;
                    case 11:
                        current = item.getNovember();
                        break;
                    case 12:
                        current = item.getDecember();
                        break;
                }
                monthAreaCount.add(MONEY.format(current));
            }
            // 区域合计
            BigDecimal areaTotal = item.getJanuary()
                    .add(item.getFebruary())
                    .add(item.getMarch())
                    .add(item.getApril())
                    .add(item.getMay())
                    .add(item.getJune())
                    .add(item.getJuly())
                    .add(item.getAugust())
                    .add(item.getSeptember())
                    .add(item.getOctober())
                    .add(item.getNovember())
                    .add(item.getDecember());;
            areaSum.add(MONEY.format(areaTotal));
        });
        result.add(areaSum);

        // 2. 添加右侧合计列
        areaColHeader.add("合计");
        for (int i = 1; i < result.size(); i++) {
            List<String> row = result.get(i);
            BigDecimal total = ZERO;
            for (int j = 1; j < row.size(); j++) {
                total = total.add(new BigDecimal(row.get(j).replaceAll(",", "")));
            }
            row.add(MONEY.format(total));
        }

        return result;
    }

    private List<List<String>> getMonthColList(List<LiftMonthCount> amountAll) {
        List<List<String>> result = Lists.newArrayList();
        if (CollectionUtils.isEmpty(amountAll)){
            return result;
        }
        List<String> header = Lists.newArrayList("区域", "1月", "2月", "3月", "4月", "5月", "6月", "7月", "8月", "9月", "10月", "11月", "12月", "合计", "占比");
        result.add(header);

        List<BigDecimal> sum = Lists.newArrayList();
        amountAll.forEach(item -> {
            // 1. 添加各区域每月的数据
            List<BigDecimal> list = Lists.newArrayList(item.getJanuary(), item.getFebruary(), item.getMarch(), item.getApril(), item.getMay(),
                    item.getJune(), item.getJuly(), item.getAugust(), item.getSeptember(), item.getOctober(), item.getNovember(),
                    item.getDecember());
            BigDecimal total = list.stream().reduce(ZERO, BigDecimal::add);
            List<String> row = Lists.newArrayList(item.getKey());
            list.forEach(amount -> row.add(MONEY.format(amount)));
            row.add(MONEY.format(total));
            list.add(total);

            // 2. 计算底部合计
            for (int i = 0; i < list.size(); i++) {
                if (sum.size() < list.size()){
                    sum.add(list.get(i));
                } else {
                    sum.set(i, DecimalUtils.add(sum.get(i), list.get(i)));
                }
            }

            result.add(row);
        });
        List<String> footer = Lists.newArrayList("合计");
        footer.addAll(sum.stream().map(amount -> MONEY.format(amount)).collect(Collectors.toList()));
        result.add(footer);

        // 3. 计算各区域比重
        BigDecimal total = sum.get(sum.size() - 1);
        for (int i = 1; i < result.size() - 1; i++) {
            List<String> row = result.get(i);
            BigDecimal ratio = new BigDecimal(row.get(row.size() - 1).replaceAll(",", EMPTY)).multiply(new BigDecimal(100)).divide(total, 2, BigDecimal.ROUND_HALF_UP);
            row.add(ratio.toString() + "%");
        }
        return result;
    }

    @Override
    public boolean contractSigningStatisticExport(TqContractDto params) {
        ContractSigningStatistic contractSigningStatistic = contractSigningStatistic(params);

        // 返回生成的excel文件
        HttpServletResponse response =ServletUtils.getResponse();
        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        response.setCharacterEncoding("utf-8");
        try (ExcelWriter excelWriter = EasyExcel.write(response.getOutputStream()).build()) {
            excelWriter.write(contractSigningStatistic.getAreaCountStatistics(), EasyExcel.writerSheet(contractSigningStatistic.getAreaCountStatisticsTitle()).build());
            excelWriter.write(contractSigningStatistic.getMonthCountStatistics(), EasyExcel.writerSheet(contractSigningStatistic.getMonthCountStatisticsTitle()).build());
            excelWriter.write(contractSigningStatistic.getWeekCountStatistics(), EasyExcel.writerSheet(contractSigningStatistic.getWeekCountStatisticsTitle()).build());
            excelWriter.write(contractSigningStatistic.getSigningAreaColStatistics(), EasyExcel.writerSheet(contractSigningStatistic.getSigningAreaColStatisticsTitle()).build());
            excelWriter.write(contractSigningStatistic.getProductionAreaColStatistics(), EasyExcel.writerSheet(contractSigningStatistic.getProductionAreaColStatisticsTitle()).build());
            excelWriter.write(contractSigningStatistic.getDeliveryMonthColStatistics(), EasyExcel.writerSheet(contractSigningStatistic.getDeliveryMonthColStatisticsTitle()).build());
            excelWriter.write(contractSigningStatistic.getProductionMonthColStatistics(), EasyExcel.writerSheet(contractSigningStatistic.getProductionMonthColStatisticsTitle()).build());
        } catch (IOException e) {
            e.printStackTrace();
            throw new ServiceException("导出失败");
        }
        return true;
    }
}
