package com.ruoyi.project.system.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.ruoyi.project.system.domain.SysUser;
import java.util.List;

import com.ruoyi.project.system.domain.vo.SysUserNickName;
import org.apache.ibatis.annotations.Param;

/**
 * 用户表 数据层
 *
 * @author ruoyi
 */
public interface SysUserMapper extends BaseMapper<SysUser> {
    /**
     * 根据条件分页查询用户列表
     *
     * @param sysUser 用户信息
     * @return 用户信息集合信息
     */
    List<SysUser> selectUserList(SysUser sysUser);

    /**
     * 根据条件分页查询用户列表
     *
     * @param user 用户信息没有admin
     * @return 用户信息集合信息
     */
    List<SysUser> selectUserListWithoutAdmin(SysUser user);

    /**
     * 根据条件分页查询已配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    List<SysUser> selectAllocatedList(SysUser user);

    /**
     * 根据条件分页查询未分配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    List<SysUser> selectUnallocatedList(SysUser user);

    /**
     * 通过用户名查询用户
     *
     * @param userName 用户名
     * @return 用户对象信息
     */
    SysUser selectUserByUserName(String userName);

    /**
     * 通过用户ID查询用户
     *
     * @param userId 用户ID
     * @return 用户对象信息
     */
    SysUser selectUserById(Long userId);

    /**
     * 新增用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    int insertUser(SysUser user);

    /**
     * 修改用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    int updateUser(SysUser user);

    /**
     * 修改用户头像
     *
     * @param userName 用户名
     * @param avatar   头像地址
     * @return 结果
     */
    int updateUserAvatar(@Param("userName") String userName, @Param("avatar") String avatar);

    /**
     * 重置用户密码
     *
     * @param userName 用户名
     * @param password 密码
     * @return 结果
     */
    int resetUserPwd(@Param("userName") String userName, @Param("password") String password);

    /**
     * 通过用户ID删除用户
     *
     * @param userId 用户ID
     * @return 结果
     */
    int deleteUserById(Long userId);

    /**
     * 批量删除用户信息
     *
     * @param userIds 需要删除的用户ID
     * @return 结果
     */
    int deleteUserByIds(Long[] userIds);

    /**
     * 校验用户名称是否唯一
     *
     * @param userName 用户名称
     * @return 结果
     */
    SysUser checkUserNameUnique(String userName);

    /**
     * 校验手机号码是否唯一
     *
     * @param phonenumber 手机号码
     * @return 结果
     */
    SysUser checkPhoneUnique(String phonenumber);

    /**
     * 校验email是否唯一
     *
     * @param email 用户邮箱
     * @return 结果
     */
    SysUser checkEmailUnique(String email);

    /**
     * 根据角色、地区、代理商查询用户
     *
     * @param roleKey       角色
     * @param areaId        地区
     * @param parentAreaId  上级地区
     * @param agentId       代理商
     * @param parentAgentId 上级代理商
     * @return 用户列表
     */
    List<SysUser> selectUserByRoleKeyAreaIdAgentId(@Param("roleKey") String roleKey,
                                                   @Param("areaId") Long areaId,
                                                   @Param("parentAreaId") Long parentAreaId,
                                                   @Param("agentId") Long agentId,
                                                   @Param("parentAgentId") Long parentAgentId);



    List<SysUser> selectNextUser(@Param("roleKeys") List<String> nextRoleKeys,
                                 @Param("areaIds") List<Long> areaIds,
                                 @Param("parentAreaId") Long parentAreaId,
                                 @Param("agentId") Long agentId,
                                 @Param("parentAgentId") Long parentAgentId, 
                                 @Param("parentSalerId") Long parentSalerId);
    
    
    List<SysUser> selectUserByIds(@Param("ids") List<Long> ids);

    List<SysUser> selectUserByDeptId(@Param("deptId") Long deptId, @Param("roles") List<String> roles);


    List<SysUserNickName> getNickNameList();

    List<SysUser> selectUserByAreaId(Long areaId);
    
    List<SysUser> selectUserByAreaIds(List<Long> areaIds);

    Long selectDeptIdById(Long userId);

    Integer existUserByAgentIds(Long[] agentIds);

    List<SysUser> selectAllUserListForOwner(@Param("roles") List<String> roles);

    List<SysUser> selectUserByRoleKeyAreaIds(@Param("roleKey") String roleKey, @Param("areaIds") List<Long> areaIds);

    /**
     * 根据角色、区域 permission 获取区域链路下的销售经理
     *
     * @param roleKey        角色
     * @param permissionChar 区域权限字符
     * @return
     */
    List<SysUser> selectSalasByAgentAreaPermission(@Param("roleKey") String roleKey,
        @Param("permissionChar") String permissionChar);

    List<SysUser> listByNickNameList(@Param("nickNameList") List<String> nickNameList);
}
